@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ArgV.h,v 1.8 1999/08/16 11:37:11 lav Exp $ */
+/* $Id: ArgV.h,v 1.10 2000/02/12 17:04:38 lav Exp $ */
 
 #ifndef ARGV_H
 #define ARGV_H
@@ -28,8 +28,10 @@ class ArgV
    char **v;
    int c;
    int ind;
+   int allocated;
 
    void Init(int,const char * const *);
+   void GetRoom(int);
 
 public:
    ArgV() { Init(0,0); }
@@ -38,7 +40,7 @@ public:
    void Append(const char *);
    ArgV(const ArgV& a) { Init(a.c,a.v); }
    ArgV(int new_c,const char * const *new_v) { Init(new_c,new_v); }
-   ~ArgV() { Empty(); }
+   ~ArgV();
 
    char *Combine(int start_index=0) const;
    char *CombineQuoted(int start_index=0) const;
