@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.75 2000/09/12 13:47:48 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.77 2000/10/12 08:23:22 lav Exp $ */
 
 #include <config.h>
 
@@ -1082,32 +1082,30 @@ Glob::Glob(const char *p)
    dirs_only=false;
    files_only=false;
    match_period=true;
+   inhibit_tilde=true;
+
+   if(pattern[0]=='~')
+   {
+      char *slash=strchr(pattern,'/');
+      if(slash)
+      {
+	 *slash=0;
+	 inhibit_tilde=HasWildcards(pattern);
+	 *slash='/';
+      }
+      else
+	 inhibit_tilde=HasWildcards(pattern);;
+   }
+   if(pattern[0] && !HasWildcards(pattern))
+   {
+      // no need to glob, just unquote
+      char *u=alloca_strdup(pattern);
+      UnquoteWildcards(u);
+      add(new FileInfo(u));
+      done=true;
+   }
 }
 
-void Glob::add_force(const char *ptr,int len)
-{
-   // insert new file name into list
-   char *s=string_alloca(len+1);
-   memcpy(s,ptr,len);
-   s[len]=0;
-   list.Add(new FileInfo(s));
-}
-void Glob::add(const char *ptr,int len)
-{
-   char *s=string_alloca(len+1);
-   memcpy(s,ptr,len);
-   s[len]=0;
-
-   int flags=FNM_PATHNAME;
-   if(match_period)
-      flags|=FNM_PERIOD;
-
-   if(pattern[0]!=0
-   && fnmatch(pattern, s, flags)!=0)
-      return; // unmatched
-
-   add_force(ptr,len);
-}
 void Glob::add_force(const FileInfo *info)
 {
    // insert new file name into list
@@ -1136,7 +1134,19 @@ void Glob::add(const FileInfo *info)
    && fnmatch(pattern, s, flags)!=0)
       return; // unmatched
 
-   add_force(info);
+   if(s[0]=='~' && inhibit_tilde)
+   {
+      char *new_name=alloca_strdup2(s,2);
+      strcpy(new_name,"./");
+      strcat(new_name,s);
+      FileInfo new_info(*info);
+      new_info.SetName(new_name);
+      add_force(&new_info);
+   }
+   else
+   {
+      add_force(info);
+   }
 }
 
 bool Glob::HasWildcards(const char *s)
@@ -1190,7 +1200,7 @@ int NoGlob::Do()
       {
 	 char *p=alloca_strdup(pattern);
 	 UnquoteWildcards(p);
-	 add(p);
+	 add(new FileInfo(p));
       }
       done=true;
       return MOVED;
