@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.h,v 1.53 2000/07/30 15:54:31 lav Exp $ */
+/* $Id: FileAccess.h,v 1.55 2000/10/12 08:23:22 lav Exp $ */
 
 #ifndef FILEACCESS_H
 #define FILEACCESS_H
@@ -362,9 +362,7 @@ protected:
    bool	 dirs_only;
    bool	 files_only;
    bool	 match_period;
-   void	 add(const char *ptr,int len);
-   void	 add_force(const char *ptr,int len);
-   void	 add(const char *ptr) { add(ptr,strlen(ptr)); }
+   bool	 inhibit_tilde;
    void	 add(const FileInfo *info);
    void	 add_force(const FileInfo *info);
    virtual ~Glob();
@@ -375,6 +373,7 @@ public:
    void DirectoriesOnly() { dirs_only=true; }
    void FilesOnly() { files_only=true; }
    void NoMatchPeriod() { match_period=false; }
+   void NoInhibitTilde() { inhibit_tilde=false; }
    void SortByName() { list.SortByName(); }
 
    static bool HasWildcards(const char *);
