@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: HttpDir.cc,v 1.56 2000/08/26 19:11:24 lav Exp $ */
+/* $Id: HttpDir.cc,v 1.58 2000/10/12 08:23:22 lav Exp $ */
 
 #include <config.h>
 #include "ascii_ctype.h"
@@ -1003,15 +1003,8 @@ HttpGlob::HttpGlob(FileAccess *s,const char *n_pattern)
    else
       dir[1]=0;	  // root directory
 
-   if(pattern[0] && !HasWildcards(pattern))
-   {
-      // no need to glob, just unquote
-      char *u=alloca_strdup(pattern);
-      UnquoteWildcards(u);
-      add(u);
-      done=true;
+   if(done)
       return;
-   }
 
    if(dir[0])
    {
