@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SleepJob.cc,v 1.9 2000/03/24 18:53:24 lav Exp $ */
+/* $Id: SleepJob.cc,v 1.12 2000/09/07 18:38:36 lav Exp $ */
 
 #include <config.h>
 #include <ctype.h>
@@ -50,23 +50,24 @@ int SleepJob::Do()
    if(Done())
       return STALL;
 
-   if(waiting)
+   if(waiting_num>0)
    {
-      if(!waiting->Done())
+      Job *j=FindDoneAwaitedJob();
+      if(!j)
 	 return STALL;
       if(!repeat)
       {
-	 exit_code=waiting->ExitCode();
-	 Delete(waiting);
-	 waiting=0;
+	 exit_code=j->ExitCode();
+	 RemoveWaiting(j);
+	 Delete(j);
 	 exec=0;
 	 done=true;
 	 return MOVED;
       }
       repeat_count++;
       start_time=now;
-      exec=(CmdExec*)waiting; // we are sure it is CmdExec.
-      waiting=0;
+      exec=(CmdExec*)j; // we are sure it is CmdExec.
+      RemoveWaiting(j);
    }
 
    if(next_time.IsInfty())
@@ -91,7 +92,7 @@ int SleepJob::Do()
 	 }
 	 exec->FeedCmd(cmd);
 	 exec->FeedCmd("\n");
-	 waiting=exec;
+	 AddWaiting(exec);
 	 exec=0;
 	 return MOVED;
       }
@@ -109,7 +110,7 @@ void SleepJob::PrintStatus(int)
 {
    if(repeat)
    {
-      printf("\tRepeat count: %d\n",repeat_count);
+      printf(_("\tRepeat count: %d\n"),repeat_count);
       return;
    }
 }
@@ -215,7 +216,8 @@ Job *cmd_at(CmdExec *parent)
 }
 #undef args
 
-#ifdef MODULE
+#include "modconfig.h"
+#ifdef MODULE_CMD_SLEEP
 CDECL void module_init()
 {
    CmdExec::RegisterCommand("sleep",cmd_sleep);
