@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Speedometer.cc,v 1.6 2000/03/23 13:26:34 lav Exp $ */
+/* $Id: Speedometer.cc,v 1.9 2000/08/26 19:11:25 lav Exp $ */
 
 #include <config.h>
 #include <math.h>
@@ -82,14 +82,17 @@ const char *Speedometer::GetStr(float r)
    if(r<1)
       return "";
    if(r<1024)
+      // for translator: those are the units. This is 'byte per second'
       sprintf(buf_rate,_("%.0fb/s"),r);
    else if(r<1024*1024)
+      // for translator: This is 'kilobyte per second'
       sprintf(buf_rate,_("%.1fK/s"),r/1024.);
    else
+      // for translator: This is 'Megabyte per second'
       sprintf(buf_rate,_("%.2fM/s"),r/1024./1024.);
    return buf_rate;
 }
-const char *Speedometer::GetETAStrFromSize(long size)
+const char *Speedometer::GetETAStrFromSize(off_t size)
 {
    buf_eta[0]=0;
 
@@ -194,7 +197,7 @@ const char *Speedometer::GetStrS(float r)
       strcat(buf_rate," ");
    return buf_rate;
 }
-const char *Speedometer::GetETAStrSFromSize(long s)
+const char *Speedometer::GetETAStrSFromSize(off_t s)
 {
    GetETAStrFromSize(s);
    if(buf_eta[0])
@@ -216,7 +219,7 @@ void Speedometer::Reset()
    last_bytes=0;
 }
 
-ResDecl res_eta_terse("xfer:eta-terse",  "yes",ResMgr::BoolValidate,0);
+ResDecl res_eta_terse("xfer:eta-terse",  "yes",ResMgr::BoolValidate,ResMgr::NoClosure);
 void Speedometer::Reconfig(const char *n)
 {
    terse=res_eta_terse.Query(0);
