@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: commands.cc,v 1.107.2.1 2000/09/07 16:22:12 lav Exp $ */
+/* $Id: commands.cc,v 1.121 2000/09/23 11:34:29 lav Exp $ */
 
 #include <config.h>
 
@@ -149,10 +149,11 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 N_("Set debug level to given value or turn debug off completely.\n"
 	 " -o <file>  redirect debug output to the file.\n")},
    {"echo",    cmd_echo,   0},
-   {"exit",    cmd_exit,   N_("exit [<code>]"),
+   {"exit",    cmd_exit,   N_("exit [<code>|bg]"),
 	 N_("exit - exit from lftp or move to background if jobs are active\n\n"
 	 "If no jobs active, the code is passed to operating system as lftp\n"
-	 "termination status. If omitted, exit code of last command is used.\n")},
+	 "termination status. If omitted, exit code of last command is used.\n"
+	 "`bg' forces moving to background if cmd:move-background is false.\n")},
    {"fg",      cmd_wait,   0,"wait"},
    {"find",    cmd_find,0,
 	 N_("Usage: find [directory]\n"
@@ -223,6 +224,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -R, --reverse          reverse mirror (put files)\n"
 	 " -L, --dereference      download symbolic links as files\n"
 	 " -N, --newer-than FILE  download only files newer than the file\n"
+	 " -P, --parallel[=N]     download N files in parallel\n"
 	 " -i RX, --include RX    include matching files\n"
 	 " -x RX, --exclude RX    exclude matching files\n"
 	 "                        RX is extended regular expression\n"
@@ -282,14 +284,14 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	 " -E  delete local files after successful transfer (dangerous)\n"
 	 " -a  use ascii mode (binary is the default)\n"
 	 " -O <base> specifies base directory or URL where files should be placed\n")},
-   {"pwd",     cmd_pwd,    "pwd [-u]",
+   {"pwd",     cmd_pwd,    "pwd [-p]",
 	 N_("Print current remote URL.\n"
 	 " -p  show password\n")},
    {"queue",   cmd_queue,  0,
 	 N_("Usage: queue <command>\n"
 	 "Add the command to queue for current site. Each site has its own\n"
-	 "command queue. It is possible to queue up a running job by using\n")
-	 "command `queue wait <jobno>'.\n"},
+	 "command queue. It is possible to queue up a running job by using\n"
+	 "command `queue wait <jobno>'.\n")},
    {"quit",    cmd_exit,   0,"exit"},
    {"quote",   cmd_ls,	   N_("quote <cmd>"),
 	 N_("Send the command uninterpreted. Use with caution - it can lead to\n"
@@ -338,7 +340,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"suspend", cmd_suspend},
    {"user",    cmd_user,   N_("user <user|URL> [<pass>]"),
 	 N_("Use specified info for remote login. If you specify URL, the password\n"
-	 "will be cached for future usage.")},
+	 "will be cached for future usage.\n")},
    {"version", cmd_ver,    "version",
 	 N_("Shows lftp version\n")},
    {"wait",    cmd_wait,   N_("wait [<jobno>]"),
@@ -477,17 +479,32 @@ Job *CmdExec::builtin_cd()
 
 Job *CmdExec::builtin_exit()
 {
+   bool bg=false;
+   int code=prev_exit_code;
    if(args->count()>=2)
    {
-      if(sscanf(args->getarg(1),"%i",&prev_exit_code)!=1)
+      const char *a=args->getarg(1);
+      if(!strcmp(a,"bg"))
+	 bg=true;
+      else if(sscanf(a,"%i",&code)!=1)
       {
 	 eprintf(_("Usage: %s [<exit_code>]\n"),args->a0());
 	 return 0;
       }
    }
+   // Note: one job is this CmdExec.
+   if(!bg && top_level
+   && !(bool)ResMgr::Query("cmd:move-background",0) && NumberOfJobs()>1)
+   {
+      eprintf(_(
+	 "There are running jobs and `cmd:move-background' is not set.\n"
+	 "Use `exit bg' to force moving to background or `kill all' to terminate jobs.\n"
+      ));
+      return 0;
+   }
    while(!Done())
       RemoveFeeder();
-   exit_code=prev_exit_code;
+   exit_code=prev_exit_code=code;
    return 0;
 }
 
@@ -618,8 +635,30 @@ Job *CmdExec::builtin_open()
 
    if(!no_bm && host && (bm=lftp_bookmarks.Lookup(host))!=0)
    {
-      char *cmd=string_alloca(5+3+strlen(bm)+2+1);
-      sprintf(cmd,"open -B %s%s\n",bm,background?"&":"");
+      char *cmd=string_alloca(5+3+3+xstrlen(user)+1+xstrlen(pass)+3+xstrlen(port)+strlen(bm)+4+1);
+      strcpy(cmd,"open -B ");
+      if(user)
+      {
+	 strcat(cmd,"-u \"");
+	 unquote(cmd+strlen(cmd),user);
+	 if(pass)
+	 {
+	    strcat(cmd,",");
+	    unquote(cmd+strlen(cmd),pass);
+	 }
+	 strcat(cmd,"\" ");
+      }
+      if(port)
+      {
+	 strcat(cmd,"-p \"");
+	 unquote(cmd+strlen(cmd),port);
+	 strcat(cmd,"\" ");
+      }
+      strcat(cmd,bm);
+      if(background)
+	 strcat(cmd," &\n");
+      strcat(cmd,";\n");
+
       PrependCmd(cmd);
    }
    else
@@ -634,29 +673,20 @@ Job *CmdExec::builtin_open()
 	    cwd_history.Set(session,session->GetCwd());
 
 	    FileAccess *new_session=0;
-	    // get session with specified protocol if protocol differs
-	    if(uc.proto && strcmp(uc.proto,session->GetProto())
-	    || session->GetProto()[0]==0 // or if current session is dummy
-	    || !strcmp(session->GetProto(),"file"))
+
+	    const char *p=uc.proto;
+	    if(!p)
+	       p=ResMgr::Query("cmd:default-protocol",uc.host);
+	    if(!p)
+	       p="ftp";
+	    new_session=FileAccess::New(p,uc.host);
+	    if(!new_session)
 	    {
-	       const char *p=uc.proto;
-	       if(!p)
-		  p=ResMgr::Query("cmd:default-protocol",0);
-	       if(!p)
-		  p="ftp";
-	       new_session=FileAccess::New(p);
-	       if(!new_session)
-	       {
-		  eprintf("%s: %s%s\n",args->a0(),p,
-			   _(" - not supported protocol"));
-		  return 0;
-	       }
-	    }
-	    else
-	    {
-	       new_session=session->Clone();
-	       new_session->AnonymousLogin();
+	       eprintf("%s: %s%s\n",args->a0(),p,
+			_(" - not supported protocol"));
+	       return 0;
 	    }
+
 	    saved_session=session;
 	    session=0;
 	    ChangeSession(new_session);
@@ -839,20 +869,18 @@ Job *CmdExec::builtin_queue()
 
    if(xstrcmp(queue->queue_cwd,session->GetCwd()))
    {
-      ArgV cd("cd");
-      cd.Append(session->GetCwd());
-      cd.Append("&");
-      queue->FeedArgV(&cd);
+      queue->FeedCmd("cd ");
+      queue->FeedQuoted(session->GetCwd());
+      queue->FeedCmd(" &\n");
 
       xfree(queue->queue_cwd);
       queue->queue_cwd=xstrdup(session->GetCwd());
    }
    if(xstrcmp(queue->queue_lcwd,cwd))
    {
-      ArgV cd("lcd");
-      cd.Append(cwd);
-      cd.Append("&");
-      queue->FeedArgV(&cd);
+      queue->FeedCmd("lcd ");
+      queue->FeedQuoted(cwd);
+      queue->FeedCmd(" &\n");
 
       xfree(queue->queue_lcwd);
       queue->queue_lcwd=xstrdup(cwd);
@@ -938,7 +966,8 @@ CMD(cat)
 {
    const char *op=args->a0();
    int opt;
-   bool ascii=true;
+   bool ascii=false;
+   bool auto_ascii=true;
 
    while((opt=args->getopt("+bau"))!=EOF)
    {
@@ -946,9 +975,11 @@ CMD(cat)
       {
       case('a'):
 	 ascii=true;
+	 auto_ascii=false;
 	 break;
       case('b'):
 	 ascii=false;
+	 auto_ascii=false;
 	 break;
       case('?'):
 	 eprintf(_("Try `help %s' for more information.\n"),op);
@@ -964,8 +995,13 @@ CMD(cat)
       return 0;
    }
    CatJob *j=new CatJob(Clone(),output,args);
-   if(ascii && args->a0()[0]!='z')
-      j->Ascii();
+   if(!auto_ascii)
+   {
+      if(ascii)
+	 j->Ascii();
+      else
+	 j->Binary();
+   }
    output=0;
    args=0;
    return j;
@@ -1575,6 +1611,14 @@ CMD(wait)
    char *jn=args->getnext();
    if(jn)
    {
+      if(!strcmp(jn,"all"))
+      {
+	 parent->WaitForAllChildren();
+	 for(int i=0; i<parent->waiting_num; i++)
+	    parent->waiting[i]->Fg();
+	 exit_code=0;
+	 return 0;
+      }
       if(!isdigit(jn[0]))
       {
 	 eprintf(_("%s: %s - not a number\n"),op,jn);
@@ -1598,7 +1642,7 @@ CMD(wait)
       eprintf(_("%s: %d - no such job\n"),op,n);
       return 0;
    }
-   if(j->parent && j->parent->waiting==j)
+   if(Job::FindWhoWaitsFor(j)!=0)
    {
       eprintf(_("%s: some other job waits for job %d\n"),op,n);
       return 0;
@@ -1896,7 +1940,7 @@ CMD(bookmark)
       {
 	 const char *value=args->getnext();
 	 int flags=0;
-	 if((bool)res_save_passwords.Query(0))
+	 if((bool)res_save_passwords.Query(session->GetHostName()))
 	    flags|=session->WITH_PASSWORD;
 	 if(value==0)
 	 {
@@ -2031,7 +2075,7 @@ CMD(lpwd)
 {
    if(!parent->cwd)
    {
-      eprintf("%s: cannot get current directory\n",args->a0());
+      eprintf("%s: %s\n",args->a0(),_("cannot get current directory"));
       return 0;
    }
    printf("%s\n",parent->cwd);
