@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: complete.cc,v 1.36 2000/08/03 22:55:58 lav Exp $ */
+/* $Id: complete.cc,v 1.38 2000/10/12 08:23:23 lav Exp $ */
 
 #include <config.h>
 
@@ -531,6 +531,7 @@ static char **lftp_completion (char *text,int start,int end)
       if(rg)
       {
 	 rg->glob->NoMatchPeriod();
+	 rg->glob->NoInhibitTilde();
 	 if(type==REMOTE_DIR)
 	    rg->glob->DirectoriesOnly();
 	 for(;;)
@@ -697,6 +698,9 @@ backslash_quote (char *string)
 	  *r++ = c;
 	  break;
 	case '~':				/* tilde expansion */
+	  if (s == string)
+	    *r++ = '.', *r++ = '/';
+	  goto def;
 	case '#':				/* comment char */
 	  if(!shell_cmd)
 	    goto def;
@@ -995,7 +999,7 @@ void lftp_readline_init ()
 
    rl_completer_quote_characters = (char*)"\"";
    rl_completer_word_break_characters = (char*)" \t\n\"";
-   rl_filename_quote_characters = (char*)" \t\n\\\">;|&()*?[]";
+   rl_filename_quote_characters = (char*)" \t\n\\\">;|&()*?[]~";
    rl_filename_quoting_function = (CPFunction*)bash_quote_filename;
    rl_filename_dequoting_function = (CPFunction*)bash_dequote_filename;
    rl_char_is_quoted_p = (Function*)lftp_char_is_quoted;
