@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: ftpclass.h,v 1.67 2000/08/29 21:52:16 lav Exp $ */
+/* $Id: ftpclass.h,v 1.68 2000/10/12 07:34:11 lav Exp $ */
 
 #ifndef FTPCLASS_H
 #define FTPCLASS_H
@@ -169,7 +169,7 @@ protected:
 private:
    bool	 auth_tls_sent;
 #else
-   static const bool ftps=false; // for convenience
+   static const bool ftps; // for convenience
 #endif
 
    /* type of transfer: TYPE_A or TYPE_I */
